<?php

/**
* @file providing the service that returns the LDAP records in Plain array format.
*
*/

namespace  Drupal\ldap_search\Services;

use Drupal\ldap_servers\Entity\Server;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\ldap_authentication\AuthenticationServers;


class SearchResources {
	use StringTranslationTrait;
	/**
	 * The main server to work with.
	 *
	 * @var \Drupal\ldap_servers\Entity\Server
	 */
	private $ldap_server;

	/**
	 * LDAP Bridge.
	 *
	 * @var \Drupal\ldap_servers\LdapBridge
	 */
	private $ldapBridge;

	/**
	 * Symfony Ldap.
	 *
	 * @var \Symfony\Component\Ldap\Ldap
	*/
	private $ldap;

	/**
	 * The search base. If NULL, we use default one. should not be esacaped.
	 *
	 * @var 
	 */
	protected $base_dn;

	/**
	 * The extra search filter. such as sAMAccountName=jbarclay.  
	 * Attribute values (e.g. jbarclay) should be esacaped before calling
	 * If NULL, we use default one.
	 *
	 * @var 
	 */
	protected $extra_filter;

	/**
	 * The type of members to include in search.
	 *   'active' 	: searches only active users
	 *   'inactive' : searches only inactive users
	 *   'both'		: 
	 *    NULL 		: Skips the Parameter altogether so searches all(active and inactive) users
	 *
	 * @var 
	 */
	protected $type;

	/**
	 * No. of records to return.
	 * @Any_Numeric_Value : Returns given no. of Records.
	 * 0 (Zero) for no limit i.e. All Records !
	 *
	 * @var 
	 */
	protected $result_count;

	/**
	 * An array of desired attributes. If omitted, we only return "mail".
	 *
	 * @var 
	 */
	protected $attributes;

	/**
	 * Entity Storage.
	 *
	 * @var \Drupal\Core\Entity\EntityStorageInterface
	*/
	protected $storage;

	/**
	 * Config.
	 *
	 * @var \Drupal\Core\Config\ImmutableConfig
	*/
	protected $config;

	/**
	 * Constructor.
	 * 
     * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   	 *   Entity type manager.
   	 * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   	 *   Config factory.
	 */
	public function __construct() {
		$this->base_dn = "";
		$this->extra_filter = "";
		$this->type = NULL;
		// By default only 1 record will be returned.
		$this->result_count = 1;
		// By default only 'mail' atribute be returned.
		$this->attributes = ["mail"];
	}

 	/**
   	 * Sets an LDAP base_dn for search.
	 *
	 * @param string $base_dn
	 *   The search base. Should not be esacaped.
	 */
	public function set_base_dn($base_dn = NULL) {
  	  if (isset($base_dn) || !empty($base_dn) || $base_dn != NULL) {
	    $this->base_dn = $base_dn;
	  }
	}

 	/**
   	 * Gets an LDAP base_dn for search.
	 *
	 * @return string $base_dn
	 *   The search base.
	 */
	public function get_base_dn() {
  	  return $this->base_dn;
	}

	/**
	 * Sets an LDAP extra_filter for search.
	 *
	 * @param string $extra_filter
	 *   The search filter.
	 */
	public function set_extra_filter($extra_filter = NULL) {
		if (isset($extra_filter) || !empty($extra_filter) || $extra_filter != NULL) {
	    	$this->extra_filter = $extra_filter;
	  	}
	}

	/**
	 * Gets an LDAP extra_filter for search.
	 *
	 * @return string $extra_filter
	 *   The search filter.
	 */
	public function get_extra_filter() {
		return $this->extra_filter;
	}

	/**
	 * Sets an LDAP type for search.
	 *
	 * @param string $type
	 *   The type of members to include in search.
	 */
	public function set_type($type = NULL) {
		if (isset($type) || !empty($type) || $type != NULL) {
			$this->type = $type;
		}
	}

	/**
	 * Gets an LDAP type for search.
	 *
	 * @return string $type
	 *   The type of members to include in search.
	 */
	public function get_type() {
		return $this->type;
	}

	/**
	 * Sets an LDAP result_count for search to return.
	 *
	 * @param int $result_count
	 *   No. of records to return. By default 1 (single) record will be returned.
	 *   However, 0 (Zero) is for no limit i.e. ALL Records.
	 */
	public function set_result_count($result_count = 1) {
		if (isset($result_count) && is_int($result_count)) {
			$this->result_count = $result_count;
		} else {
			// In Default case, i.e. by mistake if some one passes String, should fall back on int value.
			$this->result_count = 0;
		}
	}

	/**
	 * Gets an LDAP result_count for search to return.
	 *
	 * @return int $result_count
	 *   No. of records to return.
	 */
	public function get_result_count() {
		return $this->result_count;
	}

	/**
	 * Sets an LDAP attributes for search to return.
	 *
	 * @param string $attributes
	 *  A Comma Separated string of desired attributes.
	 */
	public function set_attributes($attributes) {
		if (isset($attributes) || !empty($attributes)) {
			$attributes = explode(",", $attributes);
			$this->attributes = array_map('trim',$attributes);
		}
	}

	/**
	 * Gets an LDAP attributes for search to return.
	 *
	 * @return string $attributes
	 *  A Comma Separated string of desired attributes.
	 */
	public function get_attributes() {
		return $this->attributes;
	}

	/**
	 * Function to return the LDAP Searched Resources.
	 *
	 * @param string $format
	 *  Format in which the result is expected, by default JSON formatted result is returned and
	 *  Plain array in else case.
	 * @return array $ldap_matches
	 *   Matching records found in LDAP or Error string in else case.
	 */
  	public function search_resource() {
  		$ldap_matches = $ldap_users = [];
  		
  		// Alternate way to get the LDAP Servers -
		// /** @var \Drupal\ldap_authentication\AuthenticationServers $authentication_servers */
		// $authentication_servers = \Drupal::service('ldap_authentication.servers');
		// if (!$authentication_servers->authenticationServersAvailable()) {
		//   $ldap_matches['error'] = $this->t('No server available !!');
		// } else {
		// 	$servers = $authentication_servers->getAvailableAuthenticationServers();
		// }

  		// Loop through servers to Load LDAP Server in $this->ldap_server.
  		$storage = \Drupal::entityTypeManager()->getStorage('ldap_server');
		$servers = $storage->getQuery()->execute();
		// @var \Drupal\ldap_servers\Entity\Server $server 
		foreach ($storage->loadMultiple($servers) as $this->ldap_server) {
			// Confirm LDAP Server Connectivity.
			$bindStatus = $this->testConnectionAndBind();

			if (is_object($this->ldap_server) && $this->ldap_server->status() && $bindStatus) {
				// Prepare $filter variable for Final Passing to Search.
				$filter = "(&(objectCategory=person)(objectClass=user)(sAMAccountType=805306368)";

				// Prepare Account Type check.
	      		$account_type_inactive = "(userAccountControl:1.2.840.113556.1.4.803:=2)";
				switch ($this->get_type()) {
					case 'active':
						$filter.= "(!$account_type_inactive)";
						break;
					case 'inactive':
						$filter.= $account_type_inactive;
						break;
					case 'both':
					default:
						break;
				}
				$filter.= empty($this->get_extra_filter())?')':$this->get_extra_filter().")";

				// Query LDAP if All 3 mandatory parameters available.
				if (!empty($this->get_base_dn()) && !empty($filter) && !empty($this->get_attributes()) && is_array($this->get_attributes())) {
					$options = [
						'maxItems' => $this->get_result_count(),
						'pageSize' => 1000,
					];
					try {
						$ldap_users = $this->ldap
						  ->query($this->get_base_dn(), $filter, $options)
						  ->execute()
						  ->toArray();

						// Loop thru found LDAP Users.
						foreach ($ldap_users as $ldap_user_key => $ldap_user) {
							// Loop thru asked attributes to prepare final array.
							foreach ($this->get_attributes() as $key => $attribute) {
								$ldap_matches[$ldap_user_key][$attribute] = $ldap_user->getAttribute($attribute)[0];
								// End of Looping thru asked attributes array.
							}
							// End of Looping thru LDAP Users.
						}
						if (empty($ldap_matches))
							$ldap_matches['error'] = $this->t('Records not found !!');
					}
					catch (LdapException $e) {
						$ldap_matches['error'] = $this->t('LDAP query exception @message', ['@message' => $e->getMessage()]);
					}
					// End of Query LDAP.
				} else {
	  				$ldap_matches['error'] = $this->t('Failed to meet Search Criteria !!');
	  			}
			} else {
  				$ldap_matches['error'] = $this->t('LDAP Server Unavailable or Server Binding Failed !!');
  			}
  			// End of loop through servers.
		}
	  	if (empty($ldap_matches)) {
	  		$ldap_matches['error'] = $this->t('Unable to locate current LDAP Server !!');
	  	}
		return $ldap_matches;
  	}

  	/**
   	 * Test the connection.
     *
     */
  	private function testConnectionAndBind() {
  		$this->ldapBridge = \Drupal::service('ldap.bridge');
	    $this->ldapBridge->setServer($this->ldap_server);
	    $bind_result = $this->ldapBridge->bind();
	    $this->ldap = $this->ldapBridge->get();

	    if ($this->ldap_server && $this->ldap_server->status() && $bind_result) {
	      return TRUE;
	    }

	    return FALSE;
  	}
}
